import math
import numpy as np

l0 = 2.0*math.pi              # laser wavelength reference length = 0.8 um
t0 = l0                       # optical cicle t0 = 2.64fs
Lx, Ly, Lz = 16.*l0, 16.*l0, 16.*l0
k0 = 2*math.pi/l0               # laser wavevector

Lsim = [Lx, Ly, Lz]           # length of the simulation
Tsim = 33*t0                 # duration of the simulation
resx, resy, resz = 64, 64, 8    # nb of cells in one laser wavelength

dx, dy, dz = l0/resx, l0/resy, l0/resz
nx, ny, nz = int(Lx/dx), int(Ly/dy), int(Lz/dz)  # 1280 1920 160

dt = 0.999/np.sqrt(1./dx**2+1./dy**2+1./dz**2)   # time resolution 0.014fs

Main(
    geometry = "3Dcartesian",
    
    interpolation_order = 2,
    
    cell_length = [dx, dy, dz],
    grid_length  = Lsim,
    
    number_of_patches = [ 64, 64, 8],
    
    timestep = dt,
    simulation_time = Tsim, 
    
    EM_boundary_conditions = [ ['silver-muller']],
    reference_angular_frequency_SI = 1.883652*1e15,        # very important to use FaTiDo

    # solve_poisson = False,
    
    #solve_relativistic_poisson = True,
    #relativistic_poisson_max_iteration = 50000, 
    
    random_seed = smilei_mpi_rank
)

#########################LP Laser Initialization########################
a0 = 0.1

# Temporal profile
laser_fwhm = 3.*t0   # duration of the laser, about 12 fs when lambda0=0.8 micron
tau = laser_fwhm/np.sqrt(2*np.log(2))
t_center = 9.*t0   # time center of the laser

def laser_time_profile(t):
	return np.exp(-((t-t_center)/tau)**2)	

# Spatial profile
waist = 2*l0    # beam waist at focus
x_foc = 12.*l0	# focus position of laser along x axis

LaserGaussian3D(
	box_side	= "xmin",
	a0	 	    = a0,
	omega		= 1.,
	focus		= [x_foc, Ly/2., Lz/2.],
	waist		= waist,
	ellipticity     = 0, 
    time_envelope	= laser_time_profile
)

################# Thin Foil Initialization ##########################
n0 = 16.         #density of the solid target

def dens_elec(x,y,z):

    # Set the density
    if x > 12*l0 and x < 16*l0: return n0

    else: return 0.0

def dens_ion(x,y,z):
    return dens_elec(x,y,z)

Species(
  name = 'elec',
  position_initialization = 'random',
  momentum_initialization = 'cold',
  ionization_model = 'none',
  particles_per_cell = 32,
  c_part_max = 1.0,
  mass = 1.0,
  charge = -1.0,
  number_density = dens_elec,
  time_frozen = 0.,
  boundary_conditions = [
		["remove", "remove"],
		["remove", "remove"], ['remove', 'remove']
	],
)


Species(
  name = 'ion',
  position_initialization = 'random',
  momentum_initialization = 'cold',
  ionization_model = 'none',
  particles_per_cell = 32,
  c_part_max = 1.0,
  mass = 1.0*1836,
  charge = +1.0,
  number_density = dens_ion,
  time_frozen = 0.,
  boundary_conditions = [
                ["remove", "remove"],
                ["remove", "remove"], ['remove', 'remove']
        ],
)


################# Test Electrons initialization################
nteon = 16.
skin_depth = np.sqrt(1./n0)/(2*np.pi)   # skin depth of the solid target, lambda0/(2*pi)*sqrt(nc/n0), about 0.025*lambda0

# density function of the test electrons: they only cover the preplasma and 3*skin_depth of the surface
def dens_teon(x,y,z):

    # Set the density
    if x>12*l0 and x < 12*l0 + 3*skin_depth and (y-8*l0)**2+(z-8*l0)**2<(4*l0)**2: return nteon

    else: return 0.0


Species(
  name = 'teon',
  position_initialization = 'random',
  momentum_initialization = 'cold',
  ionization_model = 'none',
  particles_per_cell = 1,
  c_part_max = 1.0,
  mass = 1.0,
  charge = -1.0,
  number_density = dens_teon,
  is_test = True,
  time_frozen = 0.,
  boundary_conditions = [
		["remove", "remove"],
		["remove", "remove"], ['remove', 'remove']
	],
  keep_interpolated_fields = ['Ex', 'Ey', 'Ez', 'Bx', 'By', 'Bz']    # very important to use FaTiDo
)

############### Load Balance ################################## 
LoadBalancing(
	initial_balance	=True,
)

############### Diagnosis ################################## 

globalEvery = int(t0/dt)

DiagScalar(
    every=globalEvery
)
'''
DiagFields(
    every = globalEvery,
    fields = ['Ex','Ey','Ez']
)


from numpy import s_
DiagFields(
    every = globalEvery,
    fields = ['Ex','Ey','Ez'],
    subgrid = s_[4:100:3, 5:400:10, 6:300:80]
)
DiagFields(
    every = 2*globalEvery,
    fields = ['Ex','Ey','Ez'],
    time_average = globalEvery
)


DiagProbe(
    every = 10,
    origin = [0.1*Lsim[0], 0.5*Lsim[1], 0.5*Lsim[2]],
    fields = []
)

DiagProbe(
    every = 100,
    number = [30],
    origin = [0.1*Lsim[0], 0.5*Lsim[1], 0.5*Lsim[2]],
    corners = [[0.9*Lsim[0], 0.5*Lsim[1], 0.5*Lsim[2]]],
    fields = []
)
'''

list_fields = ['Ex', 'Ey', 'Ez','Bz','Rho_elec']
DiagProbe(
    every = int(globalEvery),
    number = [nx, ny],
    origin = [0.*Lsim[0], 0.*Lsim[1], 0.5*Lsim[2]],
    corners = [
        [1.0*Lsim[0], 0.*Lsim[1], 0.5*Lsim[2]],
        [0.*Lsim[0], 1.0*Lsim[1], 0.5*Lsim[2]],
    ],
    fields = list_fields
)

# # Set a line probe to detect the reflected high-order harmonic
# # The probe start when the laser head reach the upper box plane
# step_start, step_end = int(14*t0/dt), int(Tsim/dt)
# step_every = 1
# DiagProbe(
#     every  = [step_start, step_end, step_every],
#     origin = [0.5*Lsim[0], 0.99*Lsim[1], 0.5*Lsim[2]],
#     corners = [
#         [0.5*Lsim[0], 0.01*Lsim[1], 0.5*Lsim[2]],
#     ],
#     number   = [576],
#     fields  = ['Ex', 'Ez','Ey']
# )

# step_start, step_end = int(12.5*t0/dt), int(Tsim/dt)
# step_every = 1
# DiagProbe(
#     every  = [step_start, step_end, step_every],
#     origin = [0.5*Lsim[0], 0.99*Lsim[1], 0.01*Lsim[2]],
#     corners = [
#         [0.5*Lsim[0], 0.99*Lsim[1], 0.99*Lsim[2]],
#     ],
#     number   = [1200],
#     fields  = ['Ex', 'Ez']
# )
# DiagProbe(
#     every = globalEvery,
#     number = [ny, nz],
#     origin = [0.5*Lsim[0], 0.*Lsim[1], 0.*Lsim[2]],
#     corners = [
#         [0.5*Lsim[0], 1.0*Lsim[1], 0.*Lsim[2]],
#         [0.5*Lsim[0], 0.*Lsim[1], 1.0*Lsim[2]],
#     ],
#     fields = list_fields
# )

# DiagProbe(
#     every = int(globalEvery/4.),
#     number = [nx],
#     origin = [0.*Lsim[0], 0.5*Lsim[1], 0.5*Lsim[2]],
#     corners = [[1.*Lsim[0], 0.5*Lsim[1], 0.5*Lsim[2]]],
#     fields = list_fields
# )

# Start a trace diagnostic of the test electrons
# the trace dianostic starts when the very front of the laser
# reach the target and ends the laser tail leave the target plus an optical cycle
t_trace_start = int((17*t0)/dt)
t_trace_end = int((26*t0)/dt)
period = 1
#### filter function to avoid unnessarry tracing 
## only tracing those with gamma>1.01, then gamma**2-1>0.21
#gamma_fil = 1.01
#def my_filter(particles):
#    return (particles.px)**2 + (particles.py)**2 + (particles.pz)**2 > (gamma_fil**2-1)

DiagTrackParticles(
    species = 'teon',
    every = [t_trace_start, t_trace_end, period],
    attributes = ['x','y','z','px', 'py', 'pz', 'w', 'Ex', 'Ey', 'Ez', 'Bx', 'By', 'Bz'],     # very important to use FaTiDo
    # filter = my_filter,
    flush_every = 500
)

